( cd usr/lib ; rm -rf libaudid3tag.so )
( cd usr/lib ; ln -sf libaudid3tag.so.1.0.0 libaudid3tag.so )
( cd usr/lib ; rm -rf libaudclient.so )
( cd usr/lib ; ln -sf libaudclient.so.1.0.0 libaudclient.so )
( cd usr/lib ; rm -rf libaudclient.so.1 )
( cd usr/lib ; ln -sf libaudclient.so.1.0.0 libaudclient.so.1 )
( cd usr/lib ; rm -rf libaudid3tag.so.1 )
( cd usr/lib ; ln -sf libaudid3tag.so.1.0.0 libaudid3tag.so.1 )
